package lv.lu.owljgraph.vertex;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.Rectangle;
import java.awt.geom.Rectangle2D;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.StringTokenizer;

import javax.swing.BorderFactory;
import javax.swing.JComponent;
import javax.swing.JTextArea;
import javax.swing.UIManager;

import lv.lu.owljgraph.OWLGraphConstants;

import org.jgraph.JGraph;
import org.jgraph.graph.AttributeMap;
import org.jgraph.graph.CellView;
import org.jgraph.graph.CellViewRenderer;

public class OWLClassRenderer extends JComponent implements CellViewRenderer, Serializable {

    private static final long serialVersionUID = 2174750597908394919L;

    transient protected JGraph graph;
    transient protected OWLClassView view;
    transient protected boolean hasFocus, selected, preview, opaque;
    transient protected Color defaultForeground, defaultBackground, borderColor;
    transient protected boolean suppressProperties = false;

    protected JCenteredTextArea header = null;
    protected JTextArea propertiesArea = null;

    private static final Insets insets = new Insets(5, 5, 5, 5);

    public static class JCenteredTextArea extends JTextArea {

        private static final long serialVersionUID = 3168204619770034373L;

        public void paint(Graphics g) {

            final Graphics2D g2d = (Graphics2D) g;
            final Rectangle2D r = getBounds();
            Insets insets = getInsets();
            
            FontMetrics metrics = getFontMetrics(getFont());
            double w = r.getWidth() - insets.left - insets.right;

            int y = insets.top;
            String[] strLines = splitString(getText());
            for (int i = 0; i < strLines.length; i++) {
                final Rectangle2D bounds = metrics.getStringBounds(strLines[i], g2d);
                y += bounds.getHeight();
                g2d.drawString(strLines[i], insets.left + (int) ((w - bounds.getWidth()) / 2), y);
            }
        }

        private String[] splitString(String text) {
            List<String> tokens = new ArrayList<String>();
            StringTokenizer tokenizer = new StringTokenizer(text, "\n");
            while (tokenizer.hasMoreTokens()) {
                tokens.add(tokenizer.nextToken());
            }
            String[] result = new String[tokens.size()];
            return tokens.toArray(result);
        }

    }

    /**
     * Creates an instance of <code>UMLClassRenderer</code>.
     */
    @SuppressWarnings("serial")
    public OWLClassRenderer() {
        defaultForeground = UIManager.getColor("Tree.textForeground");
        defaultBackground = UIManager.getColor("Tree.textBackground");

        setLayout(new BorderLayout());
        setEnabled(false);

        header = new JCenteredTextArea() {
            public Insets getInsets() {
                return insets;
            }
        };
        header.setLineWrap(false);
        add(header, BorderLayout.NORTH);

        propertiesArea = new JTextArea() {
            public Insets getInsets() {
                return insets;
            }
        };
        propertiesArea.setLineWrap(false);
        add(propertiesArea, BorderLayout.CENTER);
    }

    public Component getRendererComponent(JGraph graph, CellView view, boolean sel, boolean focus, boolean preview) {
        if (view instanceof OWLClassView) {
            this.view = (OWLClassView) view;
            setComponentOrientation(graph.getComponentOrientation());
            if (graph.getEditingCell() != view.getCell()) {
                String cellHeader = "";
                if (view instanceof OWLObjectPropertyView) {
                    cellHeader = OWLObjectPropertyCell.SIGNATURE;
                } else {
                    cellHeader = OWLClassCell.SIGNATURE;
                }
                String cellLabel = graph.convertValueToString(view.getCell());
                if (cellLabel != null) {
                    header.setText(cellHeader + "\n" + cellLabel);
                } else {
                    header.setText(null);
                }
            } else {
                header.setText(null);
            }

            this.graph = graph;
            this.hasFocus = focus;
            this.selected = sel;
            this.preview = preview;

            if (this.view.isLeaf()) {
                installAttributes(view);
            } else {
                resetAttributes();
            }

            return this;
        }

        return null;
    }

    protected void installAttributes(CellView view) {
        AttributeMap attributes = view.getAttributes();
        OWLClassCell cell = (OWLClassCell) view.getCell();

        borderColor = OWLGraphConstants.getBorderColor(attributes);
        if (getBorder() == null && borderColor != null) {
            setBorder(BorderFactory.createLineBorder(borderColor, 1));
        }

        Color foreground = OWLGraphConstants.getForeground(attributes);
        if (foreground == null) {
            foreground = defaultForeground;
        }

        Color background = OWLGraphConstants.getBackground(attributes);
        if (background == null) {
            background = defaultBackground;
        }

        setOpaque(OWLGraphConstants.isOpaque(attributes));
        setBackground(background);

        Font font = OWLGraphConstants.getFont(attributes);

        header.setOpaque(true);
        header.setFont(font);
        header.setForeground(foreground);

        propertiesArea.setFont(font);
        propertiesArea.setOpaque(false);
        propertiesArea.setText(cell.getPropertiesAreaText());
        if (propertiesArea.getText() == null || propertiesArea.getText().length() == 0) {
            suppressProperties = true;
        } else {
            suppressProperties = false;
        }
        propertiesArea.setVisible(!suppressProperties);
    }

    protected void resetAttributes() {
        header.setText(null);
        setBorder(null);
        setOpaque(false);
    }

    public void paint(Graphics g) {
        Graphics2D g2d = (Graphics2D) g;

        if (view instanceof OWLObjectPropertyView) {
            g2d.setStroke(OWLGraphConstants.OWL_OBJECT_PROPERTY_STROKE);
        }
        if (selected || hasFocus) {
            g2d.setStroke(OWLGraphConstants.SELECTION_STROKE);
            if (hasFocus) {
                g2d.setColor(graph.getGridColor());
            } else if (selected) {
                g2d.setColor(graph.getHighlightColor());
            }
        }
        
        Rectangle r = getBounds();
        int w = r.width;
        int h = r.height;

        if (getBackground() != null) {
            g2d.setColor(getBackground());
            g2d.fillRect(0, 0, w, h);
        }

        g2d.setColor(getBorderColor());

        if (!suppressProperties) {
            Rectangle pr = propertiesArea.getBounds();
            g2d.drawLine(0, pr.y, w, pr.y);
        }

        g2d.setColor(getForeground());

        super.paint(g2d);
    }

    public Color getBorderColor() {
        return borderColor;
    }

}